local ADDON, Package = ...
local L = Package[GetLocale()] or Package["enUS"]

local function create_window(name, parent)
  local frame = CreateFrame("Frame", name, parent)
  frame:EnableMouse(true)
  frame:Hide()
  frame:SetSize(360, 400)
  frame:SetPoint("CENTER")
  frame:SetMovable(true)
  frame:SetToplevel(true)
  frame:SetFrameStrata("DIALOG")
  frame:SetClampedToScreen(true)

  local background = frame:CreateTexture(nil, "BACKGROUND")
  background:SetAllPoints(frame)
  background:SetTexture([[Interface\DialogFrame\UI-DialogBox-Background]])

  local border_topleft = frame:CreateTexture(nil, "ARTWORK")
  border_topleft:SetAtlas("battlefieldminimap-border-topleft", true)
  border_topleft:SetPoint("TOPLEFT", -10, 10)

  local border_topright = frame:CreateTexture(nil, "ARTWORK")
  border_topright:SetAtlas("battlefieldminimap-border-topright", true)
  border_topright:SetPoint("TOPRIGHT", 8, 10)

  local border_bottomleft = frame:CreateTexture(nil, "ARTWORK")
  border_bottomleft:SetAtlas("battlefieldminimap-border-bottomleft", true)
  border_bottomleft:SetPoint("BOTTOMLEFT", -10, -9)

  local border_bottomright = frame:CreateTexture(nil, "ARTWORK")
  border_bottomright:SetAtlas("battlefieldminimap-border-bottomright", true)
  border_bottomright:SetPoint("BOTTOMRIGHT", 8, -9)

  local border_top = frame:CreateTexture(nil, "ARTWORK")
  border_top:SetAtlas("battlefieldminimap-border-top")
  border_top:SetPoint("TOPLEFT", border_topleft, "TOPRIGHT")
  border_top:SetPoint("BOTTOMRIGHT", border_topright, "BOTTOMLEFT")

  local border_bottom = frame:CreateTexture(nil, "ARTWORK")
  border_bottom:SetAtlas("battlefieldminimap-border-bottom")
  border_bottom:SetPoint("TOPLEFT", border_bottomleft, "TOPRIGHT")
  border_bottom:SetPoint("BOTTOMRIGHT", border_bottomright, "BOTTOMLEFT")

  local border_left = frame:CreateTexture(nil, "ARTWORK")
  border_left:SetAtlas("battlefieldminimap-border-left")
  border_left:SetPoint("TOPLEFT", border_topleft, "BOTTOMLEFT")
  border_left:SetPoint("BOTTOMRIGHT", border_bottomleft, "TOPRIGHT")

  local border_right = frame:CreateTexture(nil, "ARTWORK")
  border_right:SetAtlas("battlefieldminimap-border-right")
  border_right:SetPoint("TOPLEFT", border_topright, "BOTTOMLEFT")
  border_right:SetPoint("BOTTOMRIGHT", border_bottomright, "TOPRIGHT")

  local close_button_border = frame:CreateTexture(nil, "BORDER")
  close_button_border:SetTexture([[Interface\DialogFrame\UI-DialogBox-Corner]])
  close_button_border:SetSize(32, 32)
  close_button_border:SetPoint("TOPRIGHT", 1, 2)

  local close_button = CreateFrame("Button", nil, frame, "UIPanelCloseButton")
  close_button:SetPoint("TOPRIGHT", 5, 6)

  frame.close_button_border = close_button_border
  frame.close_button = close_button

  return frame
end

local function create_tab_button(name, parent)
  local frame = CreateFrame("Button", name, parent)
  frame:SetSize(64, 32)

  local left = frame:CreateTexture(nil, "BACKGROUND")
  left:SetSize(16, 32)
  left:SetPoint("TOPLEFT")
  left:SetTexture([[Interface\ChatFrame\ChatFrameTab]])
  left:SetTexCoord(0.0, 0.25, 0.0, 1.0)

  local middle = frame:CreateTexture(nil, "BACKGROUND")
  middle:SetSize(44, 32)
  middle:SetPoint("LEFT", left, "RIGHT")
  middle:SetTexture([[Interface\ChatFrame\ChatFrameTab]])
  middle:SetTexCoord(0.25, 0.75, 0.0, 1.0)

  local right = frame:CreateTexture(nil, "BACKGROUND")
  right:SetSize(16, 32)
  right:SetPoint("LEFT", middle, "RIGHT")
  right:SetTexture([[Interface\ChatFrame\ChatFrameTab]])
  right:SetTexCoord(0.75, 1.0, 0.0, 1.0)

  local highlight = frame:CreateTexture(nil, "BACKGROUND")
  highlight:SetPoint("LEFT", left, 0, -7)
  highlight:SetPoint("RIGHT", right, 0, -7)
  highlight:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Tab-Highlight]])
  highlight:SetBlendMode("ADD")

  local pushed = frame:CreateTexture(nil, "BACKGROUND")
  pushed:SetPoint("LEFT", left, 0, -7)
  pushed:SetPoint("RIGHT", right, 0, -7)
  pushed:SetTexture([[Interface\PaperDollInfoFrame\UI-Character-Tab-Highlight-yellow]])
  pushed:SetBlendMode("ADD")

  local font_string = frame:CreateFontString()
  font_string:SetPoint("LEFT", left, 8, -6)
  font_string:SetPoint("RIGHT", right, -8, -6)
  font_string:SetJustifyH("CENTER")

  frame:SetNormalFontObject("GameFontNormalSmall")
  frame:SetHighlightTexture(highlight)
  frame:SetPushedTexture(pushed)
  frame:SetPushedTextOffset(0, 0)
  frame:SetFontString(font_string)

  return frame
end

local function create_tab_pane(name, parent)
  local frame = CreateFrame("Frame", name, parent)

  return frame
end

local function create_tab(name, parent, items)
  local frame = CreateFrame("Frame", parent:GetName() .. name, parent)
  frame:SetSize(parent:GetWidth(), 32)
  frame:SetPoint("TOP", 0, 36)

  local function OnClick(self)
    local parent = self:GetParent()
    local id = self:GetID()

    for at = 1, #items do
      local button = _G[parent:GetName() .. "Button" .. at]
      local pane = _G[parent:GetName() .. "Pane" .. at]

      button:SetButtonState("NORMAL", false)
      pane:Hide()

      if at == id then
        button:SetButtonState("PUSHED", true)
        pane:Show()
      end
    end
  end

  local relative_to = nil

  for at = 1, #items do
    local button = create_tab_button(frame:GetName() .. "Button" .. at, frame)
    button:SetID(at)
    button:SetText(items[at])
    button:SetScript("OnClick", OnClick)

    if relative_to then
      button:SetPoint("TOPLEFT", relative_to, "TOPRIGHT", 8, 0)
    else
      button:SetPoint("TOPLEFT", 3, 0)
    end

    local pane = create_tab_pane(frame:GetName() .. "Pane" .. at, frame)
    pane:SetID(at)
    pane:SetAllPoints(parent)

    relative_to = button
  end

  local function select(index)
    local button = _G[frame:GetName() .. "Button" .. index]
    local script = button:GetScript("OnClick")

    script(button)
  end

  return frame, select
end

local function create_button(name, parent)
  local frame = CreateFrame("Button", name, parent, "UIPanelButtonTemplate")
  frame:SetSize(80, 24)

  return frame
end

local function create_checkbox(name, parent)
  local frame = CreateFrame("CheckButton", name, parent, "UICheckButtonTemplate")

  return frame
end

local function config()
  local window = create_window(ADDON .. "ConfigWindow", UIParent)

  local tab, tab_select = create_tab("Tab", window, {L["general"], L["chat"], L["gametooltip"], L["combat"]})

  local reload_button = create_button(nil, window)
  reload_button:SetText(L["reload_ui"])
  reload_button:SetPoint("BOTTOM", 0, 8)
  reload_button:SetScript("OnClick", function (self) C_UI.Reload() end)

  local PREFS = {
    {
      "action_auto_transition",

      "aura_duration",

      "cvar_camera_distance_max_zoom_factor",

      "merchant_repair_items",
      "merchant_sell_items",

      "minimap_icon_drawer",
      "minimap_zoom_in_or_out",
      "minimap_world_map",

      "paperdoll_item_quality_color_border",

      "timealert_style",

      "trade_item_quality_color_border",
      "trade_recipient_name_with_class_and_level",
    },
    {
      "chat_sticky_raid_warning",
      "chat_unclmap_frame",
      "chat_max_history",
      "chat_arrow_keys",
      "chat_message_logger",
      "chat_copy_url_to_clipboard",
      "chat_copy_text_to_clipboard",

      "cvar_chat_class_color_override",
    },
    {
      "gametooltip_item_id",
      "gametooltip_item_level",
      "gametooltip_item_price",
      "gametooltip_spell_id",
    },
    {
      "combat_disable_portrait_message",
      "combat_disable_ui_error_message",
    }
  }

  for at, entries in pairs(PREFS) do
    local relative_to = nil
    local selected_tab_pane = _G[tab:GetName() .. "Pane" .. at]

    for _, entry in pairs(entries) do
      local checkbox = create_checkbox(nil, selected_tab_pane)
      checkbox:SetSize(24, 24)
      checkbox:SetScript("OnShow", function (self) self:SetChecked(MedusaPreludeStorage[entry]) end)
      checkbox:SetScript("OnClick", function (self) MedusaPreludeStorage[entry] = self:GetChecked() end)
      checkbox.text:SetText(L[entry])
      checkbox.text:SetWidth(360 - 48)
      checkbox.text:SetJustifyH("LEFT")

      if relative_to then
        checkbox:SetPoint("TOP", relative_to, "BOTTOM", 0, -4)
      else
        checkbox:SetPoint("TOPLEFT", 8, -8)
      end

      relative_to = checkbox
    end
  end

  local tab_button1 = _G[tab:GetName() .. "Button1"]
  tab_button1:RegisterForDrag("LeftButton")
  tab_button1:SetScript("OnDragStart", function (self) self:GetParent():GetParent():StartMoving() end)
  tab_button1:SetScript("OnDragStop", function (self) self:GetParent():GetParent():StopMovingOrSizing() end)

  tab_select(1)

  table.insert(UISpecialFrames, window:GetName())
end

Package.config = config
