local ADDON, Package = ...
local Enum = {}

function Enum.clone(t)
  local stash = {}

  for k, v in pairs(t) do
    if type(v) == "table" then
      stash[k] = Enum.clone(v)
    else
      stash[k] = v
    end
  end

  return stash
end

function Enum.omit(t, keys, depth)
  local stash = {}
  local depth = depth or 0

  for k, v in pairs(t) do
    if not Enum.is_member(keys, k) then
      if type(v) == "table" and depth > 0 then
        stash[k] = Enum.omit(v, keys, depth - 1)
      else
        stash[k] = v
      end
    end
  end

  return stash
end

function Enum.rename(t, from, to)
  local stash = {}

  for k, v in pairs(t) do
    local i = k == from and to or k

    if type(v) == "table" then
      stash[i] = Enum.rename(v, from, to)
    else
      stash[i] = v
    end
  end

  return stash
end

function Enum.merge(to, from, override)
  local stash = Enum.clone(to)

  for k, v in pairs(from) do
    if type(v) == "table" and type(stash[k]) == "table" then
      stash[k] = Enum.merge(stash[k], v, override)
    else
      if stash[k] == nil or override ~= false then
        stash[k] = v
      end
    end
  end

  return stash
end

function Enum.is_member(t, value)
  for _, entry in pairs(t) do
    if entry == value then
      return true
    end
  end

  return false
end

Package.Enum = Enum
