local ADDON, Package = ...
local Color = Package.Color
local ColorPalette = Package.ColorPalette

local function gametooltip_item(is_item_id_flagged, is_item_level_flagged, is_item_price_flagged)
  local TEXT_RIGHTS = {
    GameTooltip = GameTooltipTextRight1,
    ItemRefTooltip = ItemRefTooltipTextRight1,
    ShoppingTooltip1 = ShoppingTooltip1TextRight2,
    ShoppingTooltip2 = ShoppingTooltip2TextRight2,
  }

  local function OnTooltipSetItem(self)
    local item_name, item_link = self:GetItem()

    if item_link then
      if is_item_price_flagged then
        local _, _, _, _, _, _, _, _, _, _, item_sell_price = GetItemInfo(item_link)
        local container = GetMouseFocus()
        local item_count = 1

        if type(container) == "table" and type(container.count) == "number" then
          item_count = container.count
        end

        if item_count and item_count > 0 and item_sell_price and item_sell_price > 0 and self.shownMoneyFrames == nil then
          SetTooltipMoney(self, item_count * item_sell_price, "STATIC")
        end
      end

      if is_item_id_flagged or is_item_level_flagged then
        local item_id = GetItemInfoFromHyperlink(item_link)
        local _, _, _, item_level = GetItemInfo(item_link)

        if item_id and item_level then
          local item_id = is_item_id_flagged and Color.format(ColorPalette.GRAY, item_id)
          local item_level = is_item_level_flagged and Color.format(ColorPalette.GOLD, LEVEL .. " " .. item_level)
          local text_right = TEXT_RIGHTS[self:GetName()]
          text_right:SetFormattedText("%s %s", item_id or "", item_level or "")
          text_right:Show()
        end
      end
    end
  end

  GameTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem)
  ItemRefTooltip:HookScript("OnTooltipSetItem", OnTooltipSetItem)
  ShoppingTooltip1:HookScript("OnTooltipSetItem", OnTooltipSetItem)
  ShoppingTooltip2:HookScript("OnTooltipSetItem", OnTooltipSetItem)
end

local function gametooltip_spell_id()
  local TEXT_RIGHTS = {
    GameTooltip = GameTooltipTextRight1,
    ItemRefTooltip = ItemRefTooltipTextRight1,
  }

  local function OnTooltipSetAny(self)
    local _, spell_id = self:GetSpell()

    if spell_id then
      local spell_id = Color.format(ColorPalette.GRAY, spell_id)
      local text_right = TEXT_RIGHTS[self:GetName()]
      text_right:SetFormattedText("%s %s", spell_id, text_right:GetText() or "")
      text_right:Show()
    end
  end

  GameTooltip:HookScript("OnTooltipSetItem", OnTooltipSetAny)
  GameTooltip:HookScript("OnTooltipSetSpell", OnTooltipSetAny)
  ItemRefTooltip:HookScript("OnTooltipSetSpell", OnTooltipSetAny)
end

Package.gametooltip_item = gametooltip_item
Package.gametooltip_spell_id = gametooltip_spell_id
