local ADDON, Package = ...

local function minimap_icon_drawer(position)
  local function noop() end

  local function OnEvent(self, ...)
    self:UnregisterEvent("PLAYER_ENTERING_WORLD")
    self:SetScript("OnEvent", nil)

    local function OnDragStop(self)
      self:StopMovingOrSizing()

      position[1] = self:GetLeft()
      position[2] = self:GetBottom()
    end

    local function OnDragStart(self)
      self:StartMoving()
    end

    local children = {Minimap:GetChildren()}

    local defaults = {
      MinimapBackdrop = true,
      MiniMapLFGFrame = true,
      MiniMapMailFrame = true,
      MiniMapBattlefieldFrame = true,
      TimeManagerClockButton = true,
    }

    local button = CreateFrame("CheckButton", ADDON .. "MinimapIconDrawer", UIParent)
    button:RegisterForDrag("LeftButton")
    button:EnableMouse(true)
    button:SetSize(32, 32)
    button:SetPoint("BOTTOMLEFT", position[1], position[2])
    button:SetMovable(true)
    button:SetFrameStrata("DIALOG")
    button:SetClampedToScreen(true)
    button:SetHighlightTexture([[Interface\Minimap\UI-Minimap-ZoomButton-Highlight]], "ADD")
    button:SetScript("OnClick", function (self)
      for _, frame in pairs(self.icons) do
        frame:SetShown(self:GetChecked())
      end
    end)
    button:SetScript("OnDragStart", OnDragStart)
    button:SetScript("OnDragStop", OnDragStop)
    button.icons = {}

    local border = button:CreateTexture(nil, "ARTWORK")
    border:SetTexture([[Interface\Minimap\MiniMap-TrackingBorder]])
    border:SetSize(52, 52)
    border:SetPoint("TOPLEFT", 1, -1)

    local length = button:CreateFontString(nil, "ARTWORK", "GameFontNormal")
    length:SetSize(20, 20)
    length:SetPoint("CENTER")
    length:SetJustifyH("CENTER")
    length:SetJustifyV("MIDDLE")

    local background = button:CreateTexture(nil, "BACKGROUND")
    background:SetTexture([[Interface\Minimap\UI-Minimap-Background]])
    background:SetSize(20, 20)
    background:SetPoint("CENTER")

    local relative_to = button

    for _, frame in pairs(children) do
      if not defaults[frame:GetName()] and frame:GetName():match("LibDBIcon") then
        frame:Hide()
        frame:ClearAllPoints()
        frame:SetPoint("RIGHT", relative_to, "LEFT")

        frame.Show = noop
        frame.Hide = noop
        frame.SetPoint = noop

        table.insert(button.icons, frame)

        relative_to = frame
      end
    end

    length:SetText(#button.icons)
  end

  local agent = CreateFrame("Frame")
  agent:RegisterEvent("PLAYER_ENTERING_WORLD")
  agent:SetScript("OnEvent", OnEvent)
end

local function minimap_zoom_in_or_out()
  local function OnMouseWheel(self, delta)
    local zoom = self:GetZoom() + delta
    local zoom_levels = self:GetZoomLevels() - 1

    self:SetZoom(min(zoom_levels, max(0, zoom)))
  end

  Minimap:EnableMouse(true)
  Minimap:EnableMouseWheel(true)
  Minimap:SetScript("OnMouseWheel", OnMouseWheel)

  MinimapZoomIn:Hide()
  MinimapZoomOut:Hide()
end

local function minimap_world_map()
  local function OnEvent(self, ...)
    MiniMapWorldMapButton:Hide()
  end

  local agent = CreateFrame("Frame")
  agent:RegisterEvent("PLAYER_ENTERING_WORLD")
  agent:SetScript("OnEvent", OnEvent)
end

Package.minimap_icon_drawer = minimap_icon_drawer
Package.minimap_zoom_in_or_out = minimap_zoom_in_or_out
Package.minimap_world_map = minimap_world_map
