local ADDON, Package = ...

local function paperdoll_item_quality_color_border()
  local UNITS = {player = "Character", target = "Inspect"}
  local SLOTS = {
    "Head",
    "Neck",
    "Shoulder",
    "Back",
    "Chest",
    "Shirt",
    "Tabard",
    "Wrist",
    "Hands",
    "Waist",
    "Legs",
    "Feet",
    "Finger0",
    "Finger1",
    "Trinket0",
    "Trinket1",
    "MainHand",
    "SecondaryHand",
    "Ranged",
    "Ammo"
  }

  local function OnAny()
    for unit_id, unit_frame_type in pairs(UNITS) do
      for _, item_slot_type in pairs(SLOTS) do
        local item_frame_name = unit_frame_type .. item_slot_type .. "Slot"
        local item_frame = _G[item_frame_name]
        local item_border_name = item_frame_name .. "Border"
        local item_border = _G[item_border_name]

        if item_frame and not item_border then
          local item_border_size = item_slot_type == "Ammo" and 47 or 65

          item_border = item_frame:CreateTexture(item_border_name, "OVERLAY", nil, -1)
          item_border:SetSize(item_border_size, item_border_size)
          item_border:SetPoint("CENTER")
          item_border:SetTexture([[Interface\Buttons\UI-ActionButton-Border]])
          item_border:SetBlendMode("ADD")
        end

        local item_slot_id = GetInventorySlotInfo(item_slot_type:upper() .. "SLOT")
        local item_quality = GetInventoryItemQuality(unit_id, item_slot_id)
        local item_quality_color = ITEM_QUALITY_COLORS[item_quality]

        if item_border then
          if item_quality and item_quality_color then
            item_border:SetVertexColor(item_quality_color.r, item_quality_color.g, item_quality_color.b)
            item_border:Show()
          else
            item_border:Hide()
          end
        end
      end
    end
  end

  local agent = CreateFrame("Frame")
  agent:RegisterEvent("UPDATE_INVENTORY_DURABILITY")
  agent:RegisterEvent("INSPECT_READY")
  agent:SetScript("OnEvent", OnAny)

  PaperDollFrame:HookScript("OnShow", OnAny)
end

Package.paperdoll_item_quality_color_border = paperdoll_item_quality_color_border
